#ifndef BITFIELDS_H
#define BITFIELDS_H

#include <sys/param.h>
#ifndef DARWINBUILD
 #include <endian.h>		/* Use the system's endian header */
#endif

// Bit Field Macros
// 
// These macros allow for the correct setting and extracting of bit fields
// from the given size word.
#if __BYTE_ORDER == __LITTLE_ENDIAN
#define SWAB32(word) (word)
#define SWAB64(word) (word)
#elif __BYTE_ORDER == __BIG_ENDIAN
#define SWAB32(word) ( (((word) & 0xFF000000) >> 24) | (((word) & 0x00FF0000) >> 8) | (((word) & 0x0000FF00) << 8 | (((word) & 0x000000FF) << 24)) )
#define SWAB64(word) ( SWAB32( (uint32_t)((word)>>32) ) | (uint64_t)SWAB32( (uint32_t)(word) )<<32 )
#endif

// #if __BYTE_ORDER == __LITTLE_ENDIAN
// #warning "Using Little Endian Byte Order for Unpacking"
#define BITFIELDGET(block, mask, shift)  ((SWAB32(block )& mask) >> shift)
#define BITFIELDSET(block, data, mask, shift)  block = SWAB32((SWAB32(block) & ~mask)| ((data << shift ) & mask))




#endif /* BITFIELDS_H */
