#ifndef RAWMICROBLOCKHEADERV1_H
#define RAWMICROBLOCKHEADERV1_H
#include "DAQDataFormats/RawMicroBlockHeaderV0.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMicroBlockHeader
#define  CURRENT_CLASS_VERSION        1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawmicroblockhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

namespace  VERSION_NAMESPACE {
            namespace CURRENT_FIELDS_SUB_NAMESPACE {
        
            static const uint32_t RAWMICROBLOCKHEADER_SIZE          = 2;
            static const uint32_t RAWMICROBLOCKHEADER_SIZE_PLUS_ONE = RAWMICROBLOCKHEADER_SIZE + 1;
            
            enum MicroBlockHeaderMASKS{
                MASK_V1         = 0x0000FFFF, //<! Second Version bitbield ( 16 bits)
                MASK_DestBuffID = 0x03FF0000  //<! Dest Buffer ID Mask     ( 10 bits)
            };

            enum MicroBlockHeaderWORDS{
                WORD_V1         = 1,
                WORD_DestBuffID = 1
            };

            DEFINE_SHIFT_FROM_MASK(V1);
            DEFINE_SHIFT_FROM_MASK(DestBuffID);
    }// end of namespace rawmicroblockhdr


  class RawMicroBlockHeader: public PREVIOUS_VERSION_CLASS_NAME {

        public:
            // Constructors
            RawMicroBlockHeader(const version_t version);
            RawMicroBlockHeader(const RawMicroBlockHeader&);
            ~RawMicroBlockHeader();

            using    version0::RawMicroBlockHeader::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const {return CURRENT_FIELDS_SUB_NAMESPACE::RAWMICROBLOCKHEADER_SIZE;}

            uint32_t getVersion()  const;
            bool     setVersion  (uint32_t);

            IMPLEMENT_GET_AND_SET      (DestBuffID, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(V1        , uint32_t)
  };


}} // END NAMESPCE daqdataformats
#endif /* RAWMICROBLOCKHEADERV1_H */
