#ifndef RAWNANOSLICEV2_H
#define RAWNANOSLICEV2_H
#include "DAQDataFormats/RawNanoSliceV1.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSlice
#define  CURRENT_CLASS_VERSION        2
#define  CURRENT_FIELDS_SUB_NAMESPACE rawnano
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

  namespace  VERSION_NAMESPACE {
        namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t RAWNANOSLICE_SIZE = 6;

            enum NanoSliceWORDS{
                                WORD_Value2 = 2
            };// end of NanoSliceMASKS

            enum NanoSliceMASKS{
                                MASK_Value2 = 0x0000FFFF
            };// end of NanoSliceMASKS

            DEFINE_SHIFT_FROM_MASK(Value2);
        };// end of namespace rawnano

  //! Class to hold the data from the FEBs in correct formats
  /*!
    The RawNanoSlice class is designed to mimic the format of the actual data
    block that the NOvA Front End6
    Boards (FEBS) produce when they detect a signal
    above the zero suppression threshold.
    
    The class supports general initialization with or with out data blocks as
    well as copyin style initialization for use with STL containers.
    
    Pack and unpack methods for the data rely on two types of structured data
    blocks that are provided in association with the class.
    
    The NanoSliceBlock structure breaks the raw data block out into its related
    high and low half words to give the headers, time stamps and pulseheight information
    
    The NanoSliceHeader structure further breaks out the header blocks into the
    individual data information parts.
  */

  class RawNanoSlice: public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
      
            // Constructors
            RawNanoSlice (const version_t );
            RawNanoSlice (const NanoSlice&);
            RawNanoSlice (const RawNanoSlice&);
            /// Destructor
            ~RawNanoSlice();

            /// Initialization of the Data members
            void     init();
            void     initWithData(const NanoSlice&);
            uint32_t sizeofdata  () const { return version2::rawnano::RAWNANOSLICE_SIZE;}

            //using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print(std::ostream& os=std::cout) const;

            /// Set method for all parameters of the NanoSlice
            bool     setParameters (const NanoSlice&);

            uint32_t getValue1()               const {GENERATE_FUNCTION_IS_NOT_ALLOWED; return 0;}
            bool     setValue1(const uint32_t)       {return GENERATE_FUNCTION_IS_NOT_ALLOWED;}

            INIT_FIRST(getValue, const, uint32_t, const uint32_t);
            INIT_FIRST(setValue,      , bool    , const uint32_t, const uint32_t);

        protected:

            /// Returns pointer to the word, also mask and shift
            uint32_t* getWordPointer(const uint32_t sample_number, uint32_t& mask, uint32_t& shift) const;



  };// end of RawNanoSlice
  
  
}} // END NAMESPCE daqdataformats
#endif /* RAWNANOSLICEV1_H */
