#ifndef RAWRUNHEADER_H
#define RAWRUNHEADER_H

#include "DAQDataFormats/RawRunHeaderV0.h"

// It is requred to
//   1. include DAQDataFormats/DataFormatFirst.inc
//   2. define CURRENT_CLASS
//   3.    define CURRENT_CLASS_VERSION for version of the class
//      OR define LAST_CLASS_VERSION    for the main structure of the class
//   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
//   5. include DAQDataFormats/DataFormatLast.inc
// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawRunHeader
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawrunhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        static const version_t RAWRUNHEADER_LATEST_VERSION = LAST_CLASS_VERSION;
    }

  class RawRunHeader: public PREVIOUS_VERSION_CLASS_NAME {

        public:

            RawRunHeader(const version_t version = CURRENT_FIELDS_SUB_NAMESPACE::RAWRUNHEADER_LATEST_VERSION); //!< Constructor
            RawRunHeader(const RawRunHeader& copy_in);	    //!< Copyin Constructor
            RawRunHeader(const void* buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);  //!< Constructor with data intialization
            ~RawRunHeader();                                    //!< Destructor

            IMPLEMENT_MAIN_STANDARD

            IMPLEMENT_GET_MAIN        (MarkerHi              , uint32_t)
            IMPLEMENT_GET_MAIN        (MarkerLo              , uint32_t)
            IMPLEMENT_GET_MAIN        (Reserved1             , uint32_t)
            IMPLEMENT_GET_MAIN        (Reserved2             , uint32_t)
            IMPLEMENT_GET_MAIN        (Version               , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunNumber             , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunType               , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(ConfigIDX             , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Partition             , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Subrun                , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(ConfigurationBlockSet , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunControlVersion     , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunControlID          , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(DataLoggerVersion     , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(DataLoggerID          , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunStartTime          , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(SubRunStartTime       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(SubRunEndTime         , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunEndTime            , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TotalEvents           , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(UniversalEventStartLo , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(UniversalEventStartHi , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(UniversalEventEndLo   , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(UniversalEventEndHi   , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(RunSize               , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerVersion        , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerCtrlID         , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerMC             , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(VTMod                 , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(ValidTriggerTypesLow  , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(ValidTriggerTypesHigh , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(ValidTriggerTypesHigh2, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerListIDX        , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TriggerPrescaleListIDX, uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(CRC                   , uint32_t)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getUniversalEventStart, const,uint64_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setUniversalEventStart, ,bool    , const uint64_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getUniversalEventEnd  , const,uint64_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setUniversalEventEnd  , ,bool    , const uint64_t)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(setTail               , ,bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(close                 , ,bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(advanceEventNumber    , ,bool)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(addValidTriggerTypesLow  , ,bool    , const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(addValidTriggerTypesHigh , ,bool    , const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(addValidTriggerTypesHigh2, ,bool    , const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(addRunSize               , ,bool    , const uint32_t)

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(CURRENT_CLASS)

        protected:
            bool             setFunctionPointers();

            IMPLEMENT_SET_MAIN        (MarkerHi              , uint32_t)
            IMPLEMENT_SET_MAIN        (MarkerLo              , uint32_t)
            IMPLEMENT_SET_MAIN        (Reserved1             , uint32_t)
            IMPLEMENT_SET_MAIN        (Reserved2             , uint32_t)
            IMPLEMENT_SET_MAIN        (Version               , uint32_t)


    };


} // END NAMESPACE daqdataformats
#endif /* RAWRUNHEADER_H */
