#ifndef DAQRAWTRIGGERHEADERV0_H
#define DAQRAWTRIGGERHEADERV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggerheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

  namespace CURRENT_FIELDS_SUB_NAMESPACE {
      const static uint32_t TriggerHeaderDelimiter = 0xAAAA;
  }

  namespace  VERSION_NAMESPACE {
        namespace CURRENT_FIELDS_SUB_NAMESPACE {
            static const uint32_t TRIGGERHEADER_SIZE = 6;

            enum TriggerHeaderMASKS{
                    MASK_Delimiter               = 0xFFFF0000,
                    MASK_Version                 = 0x00000FFF,
                    MASK_HWFlag                  = 0x80000000,
                    MASK_Source                  = 0x78000000,
                    MASK_SourceSubID             = 0x00FF0000,
                    MASK_SourceID                = 0x0000FFFF,
                    MASK_TriggerNumberLow        = 0xFFFFFFFF,
                    MASK_TriggerNumberHigh       = 0x000000FF,
                    MASK_MasterTriggerNumberLow  = 0xFFFFFFFF,
                    MASK_MasterTriggerNumberHigh = 0x000000FF
            };// end of TriggerHeaderMASKS
      

            enum TriggerHeaderWORDS{
                    WORD_Delimiter                  = 0,
                    WORD_Version                    = 0,
                    WORD_HWFlag                     = 1,
                    WORD_Source                     = 1,
                    WORD_SourceSubID                = 1,
                    WORD_SourceID                   = 1,
                    WORD_TriggerNumberLow           = 2,
                    WORD_TriggerNumberHigh          = 3,
                    WORD_MasterTriggerNumberLow     = 4,
                    WORD_MasterTriggerNumberHigh    = 5
                };

                DEFINE_SHIFT_FROM_MASK(Delimiter)
                DEFINE_SHIFT_FROM_MASK(Version)
                DEFINE_SHIFT_FROM_MASK(HWFlag)
                DEFINE_SHIFT_FROM_MASK(Source)
                DEFINE_SHIFT_FROM_MASK(SourceSubID)
                DEFINE_SHIFT_FROM_MASK(SourceID)
                DEFINE_SHIFT_FROM_MASK(TriggerNumberLow)
                DEFINE_SHIFT_FROM_MASK(TriggerNumberHigh)
                DEFINE_SHIFT_FROM_MASK(MasterTriggerNumberLow)
                DEFINE_SHIFT_FROM_MASK(MasterTriggerNumberHigh)
       }// end of namespace triggerheader

  //! Class to hold the data from the global trigger header in the correct format
  /*!
    The RawTriggerHeaderV0 contains the basic information required to
    reconstruct where and how the trigger was generated.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTriggerHeader: public PREVIOUS_VERSION_CLASS_NAME {
  
        public:
      
            RawTriggerHeader (const version_t); ///< The default constructor
            RawTriggerHeader (const RawTriggerHeader&);
            ~RawTriggerHeader(); ///< Destructor

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const {  return CURRENT_FIELDS_SUB_NAMESPACE::TRIGGERHEADER_SIZE;}

            /// Nanoslice of version 0 is a fixed size format
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

            IMPLEMENT_FIRST_GET_AND_SET(Delimiter              , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Version                , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(HWFlag                 , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Source                 , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(SourceID               , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(SourceSubID            , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerNumberLow       , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerNumberHigh      , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(MasterTriggerNumberLow , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(MasterTriggerNumberHigh, uint32_t)

	    INIT_FIRST(getMasterTriggerNumber, const, uint64_t, )
            INIT_FIRST(setMasterTriggerNumber,      , bool    , const uint64_t)
	    INIT_FIRST(getTriggerNumber      , const, uint64_t, )
            INIT_FIRST(setTriggerNumber      ,      , bool    , const uint64_t)
 
  };
  
  
 }} // END NAMESPCE daqdataformats
#endif /* DAQRAWTRIGGERHEADERV0_H */
