#ifndef _DBUTILS_DAQAPPMGR_APPLICATIONTYPE_H
#define _DBUTILS_DAQAPPMGR_APPLICATIONTYPE_H

#include <string>
#include <vector>

namespace dbutils {
namespace daqappmgr {

/**
 * Simple container class for the information that describes
 * a type of DAQ application.
 *
 * @author Kurt Biery
 * @version $Revision: 1.6 $ $Date: 2012/05/29 17:22:31 $
 */

class ApplicationType {

 public:

    static const std::string BNEVB_APPTYPE;
    static const std::string DL_APPTYPE;
    static const std::string DCMAPP_APPTYPE;
    static const std::string GT_APPTYPE;

    // ********************
    // *** Constructors ***
    // ********************

    ApplicationType(const std::string& typeName,
                    const std::vector<std::string>& exeNameList,
                    const std::string& startCommand,
                    const std::string& rshCommandForStart,
                    const std::string& stopCommand,
                    const std::string& rshCommandForStop,
                    const bool& supportsStatusRequest,
                    const bool& hasAppNameInProcessString);

    // *****************
    // *** Accessors ***
    // *****************

    std::string getTypeName() const {return _typeName;}
    std::vector<std::string> getExeNameList() const {return _exeNameList;}
    std::string getStartCommand() const {return _startCommand;}
    std::string getRshCommandForStart() const {return _rshCommandForStart;}
    std::string getStopCommand() const {return _stopCommand;}
    std::string getRshCommandForStop() const {return _rshCommandForStop;}
    bool supportsStatusRequest() const {return _supportsStatusRequest;}
    bool hasAppNameInProcessString() const {return _hasAppNameInProcessString;}

    // temporary (until we move to storing things in the DB)
    void setStartCommand(const std::string& cmd) {
        _startCommand = cmd;
    }
    void setRshCommandForStart(const std::string& cmd) {
        _rshCommandForStart = cmd;
    }
    void setStopCommand(const std::string& cmd) {
        _stopCommand = cmd;
    }
    void setRshCommandForStop(const std::string& cmd) {
        _rshCommandForStop = cmd;
    }

    // *****************
    // *** Operators ***
    // *****************

    bool operator==(const ApplicationType& other) const;
    bool operator!=(const ApplicationType& other) const;
    bool operator<(const ApplicationType& other) const;

 private:

    std::string _typeName;
    std::vector<std::string> _exeNameList;
    std::string _startCommand;
    std::string _rshCommandForStart;
    std::string _stopCommand;
    std::string _rshCommandForStop;
    bool _supportsStatusRequest;
    bool _hasAppNameInProcessString;
};

};  // end of namespace daqappmgr
};  // end of namespace dbutils

#endif
