#ifndef _DBUTILS_DAQCONFIG_DCMCONNECTCONFIG_H
#define _DBUTILS_DAQCONFIG_DCMCONNECTCONFIG_H

#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching and formatting of DCM
 * connection configuration data from the database.
 *
 * @author Kurt Biery
 * @version $Revision: 1.2 $ $Date: 2012/02/25 17:11:44 $
 */

class DCMConnectConfig {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    DCMConnectConfig(const std::vector<int64_t>& subsystemConfigIdList,
                     const std::string& dcmName);

    DCMConnectConfig(const SubsystemNameSpec& nameSpec,
                     const std::string& dcmName);

    DCMConnectConfig(const GlobalNameSpec& nameSpec,
                     const std::string& dcmName);

    DCMConnectConfig(const int64_t& globalConfigId,
                     const std::string& dcmName);

    virtual ~DCMConnectConfig() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}
    int64_t getPrimarySubsystemConfigId() const {return _primarySubsystemConfigId;}

 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsystemConfigIdList,
                       const std::string& dcmName);

    bool _fetchDCMApplicationParams(const std::vector<int64_t>& subsystemConfigIdList,
                                    const std::string& dcmName);

    bool _hasValidData;
    int64_t _primarySubsystemConfigId;

    // dcm-specific parameters
    int32_t _sendTimeoutInMilliSec;
    int32_t _sendNumAttempt;
    int32_t _sendSocketBufferSize;

};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
