#ifndef _DBUTILS_DAQCONFIG_PIXELTHRESHOLDS_H
#define _DBUTILS_DAQCONFIG_PIXELTHRESHOLDS_H

#include <DatabaseUtils/DAQConfig/ConfigDataTree.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching and formatting of pixel
 * threshold data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.3 $ $Date: 2011/06/22 16:16:39 $
 */

class PixelThresholds {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    PixelThresholds(const int64_t& globalConfigId,
                    std::vector<std::string> dcmList);

    virtual ~PixelThresholds() {};

    // *****************
    // *** Accessors ***
    // *****************

    bool hasValidData() const {return _hasValidData;}

    bool getThreshold(const std::string& dcmName, const int& febId,
                      const int& pixelId, int16_t& threshold);

    bool getThresholdsForFEB(const std::string& dcmName, const int& febId,
                             std::vector<int16_t>& thresholdList);

 protected:

    bool _hasValidData;
    boost::shared_ptr<ConfigDataTree> _pixelThresholdDataSet;

};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
