#ifndef __CPPUNITTESTMAIN_H_
#define __CPPUNITTESTMAIN_H_

#include <iostream>
#include <fstream>
#include <stdexcept>
#include <cppunit/TestResultCollector.h>
#include <cppunit/TestRunner.h>
#include <cppunit/TestResult.h>
#include <cppunit/TextTestProgressListener.h>
#include <cppunit/TextOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/TestSuite.h>
#include <cppunit/XmlOutputter.h>

int main ( int argc, char* argv[] ) {

  // Command line argument is test path. Defaults to "", which resolves to the top level suite.
  std::string testPath = (argc > 1) ? std::string(argv[1]) : std::string("");

  // Create the test controller
  CPPUNIT_NS::TestResult controller;
  // Add a listener that collects test results
  CPPUNIT_NS::TestResultCollector result;
  controller.addListener(&result);
  // Add a listener that prints a dot for each test as they run
  CPPUNIT_NS::TextTestProgressListener progress;
  controller.addListener(&progress);

  // Add the top suite to the test runner
  CPPUNIT_NS::TestRunner runner;
  runner.addTest(CPPUNIT_NS::TestFactoryRegistry::getRegistry().makeTest());

  try {
      std::cout << "Running "
                << (testPath.size() > 0 ? testPath.c_str() : "all tests")
                << std::endl;
    runner.run (controller, testPath);

    // Output the results as text to std::cout
    CPPUNIT_NS::TextOutputter textOutputter(&result,std::cout);
    textOutputter.write();

    // Output the results in xml format to a file
    // This is temporarily commented out until we can add user control
    // of whether this file gets generated.
    //std::ofstream xmlOutputFile("cppUnitTestResult.xml");
    //CPPUNIT_NS::XmlOutputter xmlOutputter(&result,xmlOutputFile);
    //xmlOutputter.write();
    //xmlOutputFile.close();
  }
  catch (std::invalid_argument &e) { // Test path not resolved
    std::cerr << "\n" << "ERROR: " << e.what() << std::endl;
    return 1;
  }

  return result.wasSuccessful() ? 0 : 1;
}

#endif
