#ifndef _NOVADAQ_XML_SERIALIZABLE_H
#define _NOVADAQ_XML_SERIALIZABLE_H

#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/thread/mutex.hpp>

namespace novadaq {

/**
 * The interface that is used for serializing and deserializing messages.
 *
 * @author Kurt Biery
 * @author Steve Foulkes
 * @version $Revision: 1.3 $ $Date: 2010/10/22 13:02:30 $
 */

class XMLSerializable {  // abstract class

 public:

    static const unsigned long allow_missing_elements = 0x00008000;

    virtual ~XMLSerializable() {
        return;
    }

    /**
     * Returns a string containing the serialized contents of the object.
     */
    virtual std::string serialize() const = 0;

    /**
     * Populates the internal data of the object based on serialized
     * data in the input string.
     */
    virtual void deserialize(const std::string& serializedData) = 0;

    /**
     * Returns a copy of the target object.
     */
    virtual boost::shared_ptr<XMLSerializable> clone() const = 0;

    /**
     * Returns the class name of the target object.
     */
    virtual std::string getClassName() const = 0;

 protected:

    // To avoid problems with multi-threaded access to CST XSD serialization
    // and deserialization routines (actually, the problem may just be in
    // the underlying Xerces-C++ code), we provide a static mutex in this
    // base class.  The intention is that all classes that derive from this
    // class and make use of the CST XSD routines will lock this mutex
    // thereby preventing any similar class in any other thread from accessing
    // the CST XSD code at the same time.
    static boost::mutex _serializationMutex;

};

};  // end of namespace novadaq

#endif
