//  This file (ShmRdWr.h) was created by Ron Rechenmacher <ron@fnal.gov> on
//  May 20, 2011. "TERMS AND CONDITIONS" governing this file are in the README
//  or COPYING file. If you do not have such a file, one can be obtained by
//  contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
//  $RCSfile: ShmRdWr.h,v $
//  rev="$Revision: 1.19 $$Date: 2015/01/14 19:30:16 $";
#ifndef SHMRDWR_H
#define SHMRDWR_H

#include <stdio.h>		/* printf */
#include <stdint.h>		// uint32_t
#include <stddef.h>		// offsetof
#include <netinet/in.h>         // IPPROTO_IP, sockaddr_in, htons(), htonl()
#include <unistd.h>		// SEEK_SET
#include <vector>
#include <string>

// COMPATIBILITY
#define ShmWr ShmRdWr
#define ShmRd ShmRdWr
#define start_write()		write( 0,0,true,sizeof(ShmRdWr_BufMetadata) )
#define add_data( src, sz )	write( src, sz, true )
#define shm_rdwr_buffer_header  ShmRdWr_BufMetadata
#define shm_rdwr_header		ShmRdWr_Header
#define get_ptr			getShmRdWr
#define bytes_total		data_bytes
typedef uint32_t ShmRdWrCnt;
#define check_overwrite( missed,cush )	read_grp_check( missed )
#define end_read( missed,cush )	read_grp_close( missed )
#define read_wait		read


#ifndef SEMMSL
# define SEMMSL 250		// from /usr/include/linux/sem.h
#endif

/* The format is 0xYPSSMMBB, where (all values are hex)
     Y an arbitrary, user specified value from 0 to F
     P size in "pages-1" of the semaphore user write areas (0-F is 1-0x10 pages)
    SS number of semaphore areas (or "reader groups") (from 0 to FF) actually 0 to 250
    MM main write buffers buffer size in "megabytes-1" (0-FF is 1 to 0x100 MB)
    BB number of main buffers-1 (0-FF is 1 to 0x100 buffers)
   Override this default via environment variable SHM_KEY
 */
#ifndef SHMRW_DEFAULT_KEY
#define SHMRW_DEFAULT_KEY 0x00017003 // 113 MB buffers, 4 of them
#endif

// Options
#define SHMRW_O_WRITER	1

#define SHMRW_EOVRWT	4	// arbitray value for over write error

#if 1
typedef unsigned short	shmrw_gid_t;
#else
//struct shmrw_gid_t { unsigned short gid; shmrw_gid_t(unsigned short gi):gid(gi){} inline operator unsigned short(){return gid;}};
struct shmrw_gid_t { unsigned short gid; inline operator unsigned short(){return gid;}};
#endif
#define SEMDFLT (shmrw_gid_t){0}

struct ShmRdWr_Rdr
{   volatile uint32_t	idx_rd_start;
    volatile uint32_t		idxcnt_rd_complete;
    uint32_t		total_missed; //aka total overwrite, ovrwt_tot
    uint32_t		reserved; // reserve here for naturall alignment of off_t
    off_t		offset;
    uint32_t		reserved2[2];
};
struct ShmRdWr_BufMetadata
{   size_t		data_bytes;
    timeval		wr_complete_tod;
    uint32_t		reserved[2];
};
struct ShmRdWr_Info
{   uint32_t		ovrwt;			// ovr since last read
    uint32_t		ovrwt_tot;
    int			cushion;		// current/instantaneous cushion
    timeval		last_write_tod;
    time_t		penultimate_ms;
    int			buffersAvail;
    uint8_t             *data_ptr;
    ShmRdWr_BufMetadata	bufInfo;
};
//struct ShmRdWr_Header;		// forward declaration - defined below. Need to define here for backward COMPATIBILITY
struct ShmRdWr_Header
{   uint32_t		version;
    uint16_t		bufs_num; // num_bufs=2**bufs_2n; i.e 0=1, 1=2, 2=4,...
    uint16_t    	buf_sz_MB;
    uint16_t		grpsNum;
    uint16_t		grpsSiz_pages;
    size_t		buf_sz_bytes;
    uint32_t		bufs_largest_multiple;
    uint32_t		largest_zero_offset;
    volatile uint32_t	idx_wr_start;
    volatile uint32_t	idxcnt_wr_complete; // 1st idx complete will be 1
    off_t		offset;		    // write offset corresponding to idx_wr_start
    uint32_t    	total_bytes_written;
    uint32_t		poll_period_us;
    int			semSetId;
    time_t		penultimate_ms;
    pid_t       	writer_pid;
    sockaddr_in		addr_notify_wr_complete;
};



class ShmRdWr
{
public:

    ShmRdWr(int options=0, shmrw_gid_t default_gIdx = SEMDFLT);

    static bool const semKeepAfterDataAvail/*=true*/;

    int attach();
    inline int attach( key_t k, uint32_t buf_siz, int32_t bufs_num2n ) { return attach(); } // COMPATIBILITY

    // NOTE: A "-1" (negative 1) val for off_t means "let the code manage it"
    //       A "-1" (negative 1) val for time_t mean "infinite"

    ssize_t	write( void*src, size_t, bool write_partial, off_t=-1 );
    ssize_t	write( void*src, size_t, off_t, bool write_partial=false );
    ssize_t	write( void*src, size_t );

    inline int  end_write()     { unsigned bidx=bufcnt2idx(shm_hdr_ptr_->idx_wr_start);
	return write(&shm_databuf_metadata_ptr_[bidx],sizeof(ShmRdWr_BufMetadata),(off_t)0); } // for backward COMPATIBILITY
    inline void	seek(off_t off) { off+=sizeof(ShmRdWr_BufMetadata);write(0,0,true,off); } // for backward COMPATIBILITY

    // independent/no grp/semaphore
    // bool is "allowPartial"; if true, info must be check to see if read complete (off==bytesInBuf)
    ssize_t	read( void*dest, size_t, ShmRdWr_Info&, time_t=-1, bool=false ); // [tmo] NOTE: time_t here is ms
    ssize_t	read( void*dest, size_t, uint32_t&ovrwt, time_t=-1 ); // [tmo,] NOTE: time_t here is ms
    ssize_t	read( void*dest, size_t, time_t=-1, uint&ovrwt=*(uint*)0 ); // [tmo,] NOTE: time_t here is ms

    // grp/semaphore controlled - takes sem if need be; release if no data
    // bool is "semKeepAfterDataAvail"
    // return bytes read
    ssize_t read_grp( void*dest, size_t, ShmRdWr_Info&,    time_t=-1, bool=false, shmrw_gid_t gIdx=SEMMSL ); // [tmp,grp] NOTE: time_t here is ms
    ssize_t read_grp( void*dest, size_t, uint32_t&ovrwt, time_t=-1, bool=false, shmrw_gid_t gIdx=SEMMSL ); // [tmo,grp] NOTE: time_t here is ms
    ssize_t read_grp( void*dest, size_t,   time_t=-1, bool=false, shmrw_gid_t gIdx=SEMMSL ); // [tmo, grp] NOTE: time_t here is ms

    // "start" will semKeepAfterDataAvail  (watch out for 0 byte data)
    ssize_t read_grp_start( ShmRdWr_Info&,    time_t tmo=-1, shmrw_gid_t gIdx=SEMMSL ); // return bytes available
    ssize_t read_grp_start( uint32_t&ovrwt, time_t tmo=-1, shmrw_gid_t gIdx=SEMMSL ); // return bytes available

    // for a read in-progress, (usually using ptr) _just_"check" if ovrwt has
    // occurred OR finish read, doing one last check and optionally keep semaphore
    int read_grp_check( uint32_t&ovrwt, shmrw_gid_t gIdx=SEMMSL );  // for occassional checks of overwrite
    int read_grp_close( uint32_t&ovrwt, bool=false, shmrw_gid_t gIdx=SEMMSL );  // end the read -- one last check for ovrwt, optional keepSem


    int start_read( ShmRdWr_BufMetadata**hdr, ShmRdWrCnt& missed,  uint32_t & cushion, int tmo_us ); // for backward COMPATIBILITY
    int read(  void*     dstbuf,       size_t       dstbuf_sz_bytes
	     , timeval&  this_or_last, ShmRdWrCnt& missed
	     , uint32_t& cushion,      uint32_t&    more_avail, int tmo_us=-1 ); // COMPATIBILITY


    // takes sem if need be, leaves based on arg   -- no ovrwt,etc
    int read_grp_prv( void*dest, size_t, time_t=-1, bool=false, shmrw_gid_t gIdx=SEMMSL ); // [grpIdx, tmo,release] NOTE: time_t here is ms
    int write_grp_prv( void*src, size_t, time_t=-1, bool=false, shmrw_gid_t gIdx=SEMMSL ); // [tmo,release] NOTE: time_t here is ms

    int get_grp_missed( uint32_t &missed, uint32_t &new_missed, shmrw_gid_t gIdx=SEMMSL );

    key_t getShmKey();

    inline void          setpoll_ms_( uint32_t poll_ms ) { poll_ms_ = poll_ms; }
    inline const char*   envvar()                 { return ("SHMRW_KEY"); }
    inline ShmRdWr_Header* getShmRdWr()
    {  if (shm_hdr_ptr_!=NULL) return shm_hdr_ptr_;
	else                   return (ShmRdWr_Header*)-1;
    }

    int info(  size_t   & buf_payload,    size_t   & buf_sz_bytes
	     , uint32_t & poll_period_us, uint32_t & version
	     , pid_t    & writer_pid );
    uint64_t get_us_timeofday();
    int print_header( int fd );
    friend std::ostream& operator <<( std::ostream&, ShmRdWr& );


#   ifndef TEST
private:
#   endif
    // common to reader/writer
    ShmRdWr_Header 	*shm_hdr_ptr_;

    int			 options_;
    ShmRdWr_BufMetadata	*shm_databuf_metadata_ptr_;
    uint8_t		*shm_databuf_data_ptr_;

    // read (independent) -------------------------------------------
    ShmRdWr_Rdr		rdr_info_;

    // read (group) -------------------------------------------------
    ShmRdWr_Rdr		*shm_grprdr_info__ptr_;

    uint8_t		*shm_prv_data_ptr_;
    std::vector<bool>	have_sem_;
    shmrw_gid_t		gIdx_dflt_;

    uint32_t		poll_ms_;

    int		read_next_( ShmRdWr_Info&, timeval&, ShmRdWr_Rdr& ); // returns bufIdx (>=0) OR error/tmo (<0)
    ssize_t	read_( void*,size_t,ShmRdWr_Info&, timeval&, bool, ShmRdWr_Rdr& );
    int    	getShmKeyParams(key_t&, uint16_t&,uint16_t&, uint8_t&,uint8_t&);
    size_t 	paramsToSize( uint16_t,uint16_t, uint8_t,uint8_t );
    uint32_t    bufcnt_add( uint32_t, int );
    int		bufcnt2idx( uint32_t );
    uint32_t    bufcnt_delta( uint32_t, uint32_t );
    timeval&	set_expire_time( timeval&, time_t );
    bool	is_tmo( timeval& );
    void        expire_tv2ts( const timeval&, timespec& );
    bool	semtake( shmrw_gid_t, timespec& );
    void	semgive( shmrw_gid_t );
    void	tprint( const char *fmt, ... )__attribute__((format(printf,2,3)));//attribute--recall arg shift b/c of implicit "this" arg.
};

#endif // SHMRDWR_H
