#include "DAQDataFormats/RawDataBlock.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlock
#define  LAST_CLASS_VERSION           2
#define  CURRENT_FIELDS_SUB_NAMESPACE datablock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{

//------------------------------------------------------------------------------
// Default constructor
  RawDataBlock::RawDataBlock(const uint32_t size, const uint32_t tlvl, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version, tlvl)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" start")
    reserveInternalBuffer(size);
    defaultConstructorFloatingSize();
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" end")
}

//------------------------------------------------------------------------------
RawDataBlock::RawDataBlock(const RawDataBlock& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" start")
    lastOperationsInDeepCopy(copy_in);
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" end")
}

//------------------------------------------------------------------------------
RawDataBlock::RawDataBlock(const void* datablock_buffer, const version_t version, const uint32_t buffersize):
    PREVIOUS_VERSION_CLASS_NAME(version, 0)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" start")
    reserveInternalBuffer(buffersize);
    copy(datablock_buffer);
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__<<" end")
}

//------------------------------------------------------------------------------
// Destructor
RawDataBlock::~RawDataBlock() {
    tprintf(24,"~RawDataBlock (this=%p)\n",this);
}

/// All binding routines
#define STANDARD_ROUTINES      \
            BIND_GET(Version , version0, uint32_t);\
	    BIND_GET(Marker, version0, uint32_t);\
            BIND_GET(DataSize , version0, uint32_t);\
            BIND_GET(CRC , version0, uint32_t);\
            BIND_GET_AND_SET(TrigNumLo , version0, uint32_t);\
            BIND_GET_AND_SET(TrigNumHi , version0, uint32_t);\
            BIND_GET_AND_SET(TrigNum , version0, uint64_t);\
            BIND_GET(NumMicroBlocks , version0, uint16_t);\
            BIND_GET_AND_SET(BuffId , version0, uint16_t);\
            BIND_GET_AND_SET(MonteCarloFlag , version0, bool);\
            BIND_IS(CRCCalculationUsed , version0);\
            BIND_IS_AND_SET(ConnectionInitialization , version0);\
            BIND_IS_AND_SET(CloseConnection , version0);\
            BIND_IS_AND_SET(MissingData , version0);\
	    BIND_CUSTOM_FUNCTION(setMarker, version1, , bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setFloatingMicroBlock, version0, const, bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setFloatingMicroBlockToBufferSource, version0, const, bool, const void*); \
            BIND_CUSTOM_FUNCTION(addMicroBlock1, version0,,bool, const void*, const bool); \
            BIND_CUSTOM_FUNCTION(addMicroBlock2, version0,,bool, const void*, const uint32_t, const bool); \
            BIND_CUSTOM_FUNCTION(addMicroBlock3, version0,,bool, const void*, const void*   , const version_t, const bool); \
            BIND_CUSTOM_FUNCTION(addMicroBlock4, version0,,bool, const void*, const void*   , const uint32_t , const version_t, const bool); \
	    BIND_CUSTOM_FUNCTION(addDroppedMicroblock, version1,,bool, const void*);\
	    BIND_CUSTOM_FUNCTION(addDCMData1, version2,, bool, const void*);\
	    BIND_CUSTOM_FUNCTION(addDCMData2, version2,, bool, const void*, const uint32_t); \
            BIND_CUSTOM_FUNCTION(close, version0, , bool, const bool); \
            BIND_STADARD_FUNCTION(setBufferSource); \
            BIND_CUSTOM_FUNCTION(readDataFromFileDescriptor, version0,,readDataStatus, const int);\
	    BIND_STADARD_FUNCTIONS

#include "DAQDataFormats/setFunctionPointers_template.inc"


}// end of namespace


