#include "DAQDataFormats/RawDataBlockV1.h"
#include "DAQDataFormats/RawDataBlock.h"
#include "DAQDataFormats/DataFormatException.h"

#include <sys/time.h>		// gettimeofday
#include <stdarg.h>             // va_start
#include <stdio.h>              // printf

#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlock
#define  CURRENT_CLASS_VERSION        1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawnano
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{
//------------------------------------------------------------------------------
// Default constructor
  RawDataBlock::RawDataBlock(const version_t version, const uint32_t tlvl):
    PREVIOUS_VERSION_CLASS_NAME(version, tlvl)
    ,ZEROOUT_FUNCTION(setMarker)
    ,ZEROOUT_FUNCTION(addDroppedMicroblock)
{}
//------------------------------------------------------------------------------
RawDataBlock::RawDataBlock(const RawDataBlock& copyin):
     PREVIOUS_VERSION_CLASS_NAME(copyin)
    ,DEEPCOPY_FUNCTION(setMarker)
    ,DEEPCOPY_FUNCTION(addDroppedMicroblock)
{ }

//------------------------------------------------------------------------------
// Destructor
RawDataBlock::~RawDataBlock(){
}

//------------------------------------------------------------------------------
//Getters

//------------------------------------------------------------------------------
//Setters
  bool RawDataBlock::setMarker(const uint32_t mar) { return _Header->setMarker(mar); }

//------------------------------------------------------------------------------
// Add a Dropped MicroBlock Summary into the DataBlock. Set the Summary Block marker
bool RawDataBlock::addDroppedMicroblock(const void* droppedmicroblock_buffer){
  unsigned droppedmicroblock_size = 4; // DroppedMicroblock Summary is 4 words
    PRINT_ON_DEBUG("In "<<__PRETTY_FUNCTION__<<"  0")

    PRINT_ON_DEBUG("Size of internal buffer before reading "<<_InternalBuffer.size())


    /// Before reading, we need to adjust the size of the DataBlock
    /// Cannot rely on the size adjustment from readDataGeneral, because we don't read CRC from microblock
    const uint32_t new_sizeofdata = EXECUTE_FUNCTION(sizeofdata)() + droppedmicroblock_size;

    /// If buffer is internal, we need to adjust its size
    if(_shouldBufferBeInternal) {
        /// Microblock size is the size of the header and the microslice

        /// Add the microblock size
        resizeInternalBuffer(new_sizeofdata);

        /// Check pointers after each read or resize buffer
        checkBufferInternalness();
    }// end of adjusting the internal buffer size
    
    /// Read the MicroBlock to the _Buffer starting from the previous position
    /// -1 because of CRC
    const uint32_t start_pos = EXECUTE_FUNCTION(sizeofdata)() - 1;
    readDataGeneral(droppedmicroblock_buffer, droppedmicroblock_size, start_pos);

    /// Check pointers after each read or resize buffer
    checkBufferInternalness();

    PRINT_ON_DEBUG("Size of internal buffer after reading "<<_InternalBuffer.size())

    /// Change the DataBlock header appropriately
    _Header->setDataSize(new_sizeofdata);
    _Header->advanceNumMicroBlocks();
    /// This is critical...flip the license plate on the RawDataBlock to make it a RawSummaryBlock...
    _Header->setMarker(daqdataformats::datablockheader::SummaryBlock_Marker);

    return true;
}

  daqdataformats::RawSummaryDroppedMicroblock* RawDataBlock::getDroppedMicroblock()
  {
    //The DroppedMicroblock Summary should always be first...
    void* pointer = (void*)&RAWBUFF32[_Header->sizeofdata()];
    return new daqdataformats::RawSummaryDroppedMicroblock(pointer , 0);
  }

}} // end of namespaces
