#include "DAQDataFormats/RawEventHeaderV1.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEventHeader
#define  CURRENT_CLASS_VERSION        1
#define  CURRENT_FIELDS_SUB_NAMESPACE raweventhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{
//------------------------------------------------------------------------------
// Default constructor
RawEventHeader::RawEventHeader(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(TriggerFilled)
            ,ZEROOUT_GET_AND_SET(TailFilled)
{ }

//------------------------------------------------------------------------------
RawEventHeader::RawEventHeader(const RawEventHeader& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(TriggerFilled)
            ,DEEPCOPY_GET_AND_SET(TailFilled)
{ }

//------------------------------------------------------------------------------
// Destructor
RawEventHeader::~RawEventHeader() { }


//------------------------------------------------------------------------------
// Print the MilliSlice header information
void RawEventHeader::print(std::ostream& os) const{
    os<<"\nRawEvent Header:";
    os<<boost::format(" MarkerLeft=%u"          ) % EXECUTE_FUNCTION(getMarkerLeft)();   //! MilliSlice header delimiter
    os<<boost::format(" MarkerRight=%u"         ) % EXECUTE_FUNCTION(getMarkerRight)();  //! MilliSlice header delimiter
    os<<boost::format(" Version=%u"             ) % EXECUTE_FUNCTION(getVersion)();      //! Version number for the MilliSlice event builder
    os<<boost::format(" DataLoggerID=%u"        ) % EXECUTE_FUNCTION(getDataLoggerID)();
    os<<boost::format(" RunNumber=%u"           ) % EXECUTE_FUNCTION(getRunNumber)();
    os<<boost::format(" RunType=%u"             ) % EXECUTE_FUNCTION(getRunType)();
    os<<boost::format(" ConfigIDX=%u"           ) % EXECUTE_FUNCTION(getConfigIDX)();
    os<<boost::format(" Partition=%u"           ) % EXECUTE_FUNCTION(getPartition)();
    os<<boost::format(" Subrun=%u"              ) % EXECUTE_FUNCTION(getSubrun)();
    os<<boost::format(" EventNumberLo=%u"       ) % EXECUTE_FUNCTION(getEventNumberLo)();
    os<<boost::format(" EventNumberHi=%u"       ) % EXECUTE_FUNCTION(getEventNumberHi)();
    os<<boost::format(" EventType=%u"           ) % EXECUTE_FUNCTION(getEventType)();
    os<<boost::format(" EventSize=%u"           ) % EXECUTE_FUNCTION(getEventSize)();
    os<<boost::format(" DataBlockNumber=%u"     ) % EXECUTE_FUNCTION(getDataBlockNumber)();
    os<<boost::format(" isEventIncomplete=%u"   ) % EXECUTE_FUNCTION(isEventIncomplete)();
    os<<boost::format(" MonteCarloFlag=%u"      ) % EXECUTE_FUNCTION(getMonteCarloFlag)();
    os<<boost::format(" isCRCCalculationUsed=%u") % EXECUTE_FUNCTION(isCRCCalculationUsed)();
    os<<boost::format(" isTriggerFilled=%u"     ) % EXECUTE_FUNCTION(getTriggerFilled)();
    os<<boost::format(" isTailFilled=%u"        ) % EXECUTE_FUNCTION(getTailFilled)();

    const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

    os<<"\nRAW:";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<boost::format(" ");
        printWord(i, false, os);
    }

    os<<"\nRAW:";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<boost::format("\n");
        printWord(i, true, os);
    }

}


}}
