#include "DAQDataFormats/RawSummaryDCMDataHeaderV0.h"
#include "DAQDataFormats/RawMilliSliceHeader.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryDCMDataHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE summarydcmdatahdr
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats{
namespace VERSION_NAMESPACE{
//------------------------------------------------------------
// Default constructor
RawSummaryDCMDataHeader::RawSummaryDCMDataHeader(const version_t version):
     PREVIOUS_VERSION_CLASS_NAME(version)
    ,ZEROOUT_GET_AND_SET(Version)
    ,ZEROOUT_GET_AND_SET(NumEntries)
     ,ZEROOUT_FUNCTION(advanceNumEntries)
{}

//------------------------------------------------------------
RawSummaryDCMDataHeader::RawSummaryDCMDataHeader(const RawSummaryDCMDataHeader& copyin):
     PREVIOUS_VERSION_CLASS_NAME(copyin)
    ,DEEPCOPY_GET_AND_SET(Version)
    ,DEEPCOPY_GET_AND_SET(NumEntries)
     ,DEEPCOPY_FUNCTION(advanceNumEntries)
{}

//------------------------------------------------------------
// Destructor
RawSummaryDCMDataHeader::~RawSummaryDCMDataHeader() {
}

//------------------------------------------------------------
// Initialize the data members
void RawSummaryDCMDataHeader::init(){
    EXECUTE_FUNCTION(setVersion)(_version);
    EXECUTE_FUNCTION(setNumEntries)(0);
}

bool RawSummaryDCMDataHeader::advanceNumEntries()   
        { RETURN_FUNCTION(setNumEntries)(EXECUTE_FUNCTION(getNumEntries)() + 1);}

//------------------------------------------------------------------------------
// Print the MicroBlock header information
void RawSummaryDCMDataHeader::print(std::ostream& os) const{
    const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

    os<<boost::format("\nSummaryDCMData Header(size=%u, version=%u):") % size_of_data % _version ;
    os<<boost::format(" getNumEntries=%u")% getNumEntries();
    os<<boost::format(" Version=%u"     )% EXECUTE_FUNCTION(getVersion)();
    os<<"\n";

    printBuffer(os);

    os<<"\nSummaryDCMDataHeader RAW 32bit words (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<" ";
        printWord(i,false, os);
    }

    os<<"\nSummaryDCMDataHeader RAW bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<std::endl;
        printWord(i,true, os);
    }
}


}}// end of namespaces
