#include "DAQDataFormats/RawSummaryDCMDataPointV0.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryDCMDataPoint
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE summarydcmdatapoint
#include "DAQDataFormats/DataFormatLast.inc"

//#define PRINT_ON_DEBUG(x) std::cout<<x<<"\n";

namespace daqdataformats{

namespace VERSION_NAMESPACE{
 //------------------------Constructors------------------------------------------
RawSummaryDCMDataPoint::RawSummaryDCMDataPoint(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(Version)
	     ,ZEROOUT_GET_AND_SET(DataType)
	     ,ZEROOUT_GET_AND_SET(DCMID)
            ,ZEROOUT_GET_AND_SET(TimeStampLow)
	     ,ZEROOUT_GET_AND_SET(TimeStampHigh)
{}

//------------------------------------------------------------------------------
RawSummaryDCMDataPoint::RawSummaryDCMDataPoint (const RawSummaryDCMDataPoint& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(Version)
	     ,DEEPCOPY_GET_AND_SET(DataType)
	     ,DEEPCOPY_GET_AND_SET(DCMID)
            ,DEEPCOPY_GET_AND_SET(TimeStampLow)
	     ,DEEPCOPY_GET_AND_SET(TimeStampHigh)
{
}
//------------------------Destructor--------------------------------------------
RawSummaryDCMDataPoint::~RawSummaryDCMDataPoint() {}

//------------------------------------------------------------------------------
// Default Initialization of Data members
void RawSummaryDCMDataPoint::init(){
    EXECUTE_FUNCTION(setVersion)(_version);
}

//------------------------------------------------------------------------------
// get Method for 64 bit time stamp
uint64_t RawSummaryDCMDataPoint::getTimeStamp() const          { 
    return getUint64_t(EXECUTE_FUNCTION(getTimeStampLow)(), EXECUTE_FUNCTION(getTimeStampHigh)());
}

//------------------------------------------------------------------------------
// set Method for 64bit time stamp
bool RawSummaryDCMDataPoint::setTimeStamp(const uint64_t timestamp)  {
      uint32_t low, high;
      getLoAndHi(timestamp, low, high);

      EXECUTE_FUNCTION(setTimeStampHigh)(high);
      RETURN_FUNCTION(setTimeStampLow)(low);
}

//------------------------------------------------------------------------------
// Printing the NanoSlice
void RawSummaryDCMDataPoint::print(std::ostream& os) const {

  os<<boost::format("SummaryDCMDataPoint: Verion=%u Type=%u DCMID=%u TimeStampLow=%u TimeStampHigh=%u ")
                %EXECUTE_FUNCTION(getVersion)()
                %EXECUTE_FUNCTION(getDataType)()
    %EXECUTE_FUNCTION(getDCMID)()
				  %EXECUTE_FUNCTION(getTimeStampLow)()
				  %EXECUTE_FUNCTION(getTimeStampHigh)();

  const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

  os<<"Data size = "<<size_of_data<<"\n";

  
  os<<"\nRAW(hex):";
  for(uint32_t i = 0; i < size_of_data; ++i){
    printWord(i,false, os);
    os<<" ";
  }
  
  os<<"\nRAW(binary):";
  for(uint32_t i = 0; i < size_of_data; ++i){
    if(i > 0)os<<"    ";
    printWord(i,true, os);
    os<<std::endl;
  }
}


}}// end of namespaces

