#include "DAQDataFormats/RawSummaryDroppedMicroblockV0.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryDroppedMicroblock
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE summarydroppedmicroblock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------Constructors------------------------------------------
RawSummaryDroppedMicroblock::RawSummaryDroppedMicroblock(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(Version       )
            ,ZEROOUT_GET_AND_SET(TimeStampLow  )
            ,ZEROOUT_GET_AND_SET(TimeStampHigh )
            ,ZEROOUT_GET_AND_SET(TimeWindow_sec )
            ,ZEROOUT_GET_AND_SET(TimeWindow_usec)
            ,ZEROOUT_GET_AND_SET(NumDroppedMicroblocks )
            ,ZEROOUT_GET_AND_SET(TimeStamp      )
            ,ZEROOUT_GET_AND_SET(TimeWindowTvUsec)
            ,ZEROOUT_GET_AND_SET(TimeWindowTvSec )
            ,ZEROOUT_GET_AND_SET(TimeWindowTv    )
{}

//------------------------------------------------------------------------------
RawSummaryDroppedMicroblock::RawSummaryDroppedMicroblock(const RawSummaryDroppedMicroblock& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(Version       )
            ,DEEPCOPY_GET_AND_SET(TimeStampLow  )
            ,DEEPCOPY_GET_AND_SET(TimeStampHigh )
            ,DEEPCOPY_GET_AND_SET(TimeWindow_sec )
            ,DEEPCOPY_GET_AND_SET(TimeWindow_usec)
            ,DEEPCOPY_GET_AND_SET(NumDroppedMicroblocks )
            ,DEEPCOPY_GET_AND_SET(TimeStamp      )
            ,DEEPCOPY_GET_AND_SET(TimeWindowTvUsec)
            ,DEEPCOPY_GET_AND_SET(TimeWindowTvSec )
            ,DEEPCOPY_GET_AND_SET(TimeWindowTv    )
{}

//------------------------------------------------------------------------------
RawSummaryDroppedMicroblock::~RawSummaryDroppedMicroblock(){}

//------------------------------------------------------------------------------
// General Initialization
void RawSummaryDroppedMicroblock::init(){
    EXECUTE_FUNCTION(setVersion)(_version);
}

//------------------------------------------------------------------------------
// get Method for 64 bit time stamp
uint64_t RawSummaryDroppedMicroblock::getTimeStamp() const          { 
    return getUint64_t(EXECUTE_FUNCTION(getTimeStampLow)(), EXECUTE_FUNCTION(getTimeStampHigh)());
}

//------------------------------------------------------------------------------
// set Method for 64bit time stamp
bool RawSummaryDroppedMicroblock::setTimeStamp(const uint64_t timestamp)  {
      uint32_t low, high;
      getLoAndHi(timestamp, low, high);

      EXECUTE_FUNCTION(setTimeStampHigh)(high);
      RETURN_FUNCTION(setTimeStampLow)(low);
}

//------------------------------------------------------------------------------
// get Method for the the seconds of the timeval
time_t RawSummaryDroppedMicroblock::getTimeWindowTvSec() const { return (time_t) EXECUTE_FUNCTION(getTimeWindow_sec)();}

//------------------------------------------------------------------------------
// Get Method for the the microseconds of the timeval
suseconds_t RawSummaryDroppedMicroblock::getTimeWindowTvUsec() const { return (suseconds_t) EXECUTE_FUNCTION(getTimeWindow_usec)();}

//------------------------------------------------------------------------------
bool RawSummaryDroppedMicroblock::setTimeWindowTvSec (const time_t      a) { RETURN_FUNCTION(setTimeWindow_sec)((const uint32_t)a);}
bool RawSummaryDroppedMicroblock::setTimeWindowTvUsec(const suseconds_t a) { RETURN_FUNCTION(setTimeWindow_usec)((const uint32_t)a);}

//------------------------------------------------------------------------------
// get Method for 64 bit time stamp, in timeval format
timeval RawSummaryDroppedMicroblock::getTimeWindowTv() const {

  timeval timewindow;
  timewindow.tv_usec = EXECUTE_FUNCTION(getTimeWindowTvUsec)();
  timewindow.tv_sec  = EXECUTE_FUNCTION(getTimeWindowTvSec)();

  return timewindow;
}

//------------------------------------------------------------------------------
// set Method for 64bit time stamp, as a timeval
bool RawSummaryDroppedMicroblock::setTimeWindowTv(timeval timewindow){

    EXECUTE_FUNCTION(setTimeWindowTvUsec)(timewindow.tv_usec);
    RETURN_FUNCTION(setTimeWindowTvSec)(timewindow.tv_sec);
}

//------------------------------------------------------------------------------
void RawSummaryDroppedMicroblock::print(std::ostream& os) const{
    os<<"++++ Summary Dropped Microblock ++++\n";
    os<<boost::format("Dropped Microblock Version:\t%u\n")%EXECUTE_FUNCTION(getVersion)();
    os<<boost::format("Gen Timestamp Low:\t%u\n")%EXECUTE_FUNCTION(getTimeStampLow)();
    os<<boost::format("Gen Timestamp High:\t%u\n")%EXECUTE_FUNCTION(getTimeStampHigh)();
    os<<boost::format("Gen Timestamp:\t\t%Lu\n")%(long long unsigned int)EXECUTE_FUNCTION(getTimeStamp)();
    time_t timeWindowTvSec = EXECUTE_FUNCTION(getTimeWindowTvSec)();
    char ctimestr[27]; ctime_r(&timeWindowTvSec,ctimestr);
    os<<boost::format("TimeWindow:\t\t%s")%ctimestr;
    os<<boost::format("TimeWindow usec:\t\t%d\n")%(uint32_t) EXECUTE_FUNCTION(getTimeWindowTvUsec)();
    os<<boost::format("Num Dropped Microblocks:\t\t0x%02x\n")%EXECUTE_FUNCTION(getNumDroppedMicroblocks)();
    os<<boost::format("Dropped Microblock Version:\t%d\n")%EXECUTE_FUNCTION(getVersion)();
    PRINT_ON_DEBUG("Buffer is set to "<<_Buffer)

}

}}// end of namespaces
