#include "DAQDataFormats/RawTimingMarker.h"
#include "DAQDataFormats/TimeStampCounter.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/Macros.h"

#include <boost/format.hpp>

namespace daqdataformats {

//------------------------------------------------------------------------------
// Default constructor
RawTimingMarker::RawTimingMarker():
     RawDAQData( 0
                , (const init_t)           &RawDAQData::init
                , (const sizeofdata_t)     &RawTimingMarker::sizeofdata
                , (const setBufferSource_t)&RawDAQData::setBufferSource
                , (const print_t)          &RawTimingMarker::printStandard
                , (const readData_t)       &RawTimingMarker::readDataFromBuffer
                )
{
    /// Perform general routines
    defaultConstructorFixedSize();
}
//------------------------------------------------------------------------------
RawTimingMarker::RawTimingMarker(const RawTimingMarker& copy_in):
     RawDAQData(copy_in)
{
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawTimingMarker::RawTimingMarker(const void* timingmarker_buffer):
     RawDAQData( 0
                , (const init_t)           &RawDAQData::init
                , (const sizeofdata_t)     &RawTimingMarker::sizeofdata
                , (const setBufferSource_t)&RawDAQData::setBufferSource
                , (const print_t)          &RawTimingMarker::printStandard
                , (const readData_t)       &RawTimingMarker::readDataFromBuffer
                )
{
    copy(timingmarker_buffer);
}

//------------------------------------------------------------------------------
// Destructor
RawTimingMarker::~RawTimingMarker() { }

//------------------------------------------------------------------------------
// Getters
uint32_t RawTimingMarker::getLowWord() const  { return BITFIELDGET(RAWBUFF32[timmarker::WORD_Lo], timmarker::MASK_Lo, timmarker::SHIFT_Lo);}
uint32_t RawTimingMarker::getHighWord() const { return BITFIELDGET(RAWBUFF32[timmarker::WORD_Hi], timmarker::MASK_Hi, timmarker::SHIFT_Hi);}
/**
 * Return the timestamp as a single 64-bit variable
 *
 * @return full timestamp in clock ticks
 */
uint64_t RawTimingMarker::getTime() const { return getUint64_t(getLowWord(), getHighWord()); }

//------------------------------------------------------------------------------
// Setters
bool RawTimingMarker::setLowWord (const uint32_t word) {BITFIELDSET(RAWBUFF32[timmarker::WORD_Lo], word, timmarker::MASK_Lo, timmarker::SHIFT_Lo); return true;}
bool RawTimingMarker::setHighWord(const uint32_t word) {BITFIELDSET(RAWBUFF32[timmarker::WORD_Hi], word, timmarker::MASK_Hi, timmarker::SHIFT_Lo); return true;}
/**
 * Set time using 64-bit argument
 *
 * @param timestamp as 64-bit
 */
bool RawTimingMarker::setTime(const uint64_t timestamp) {
      uint32_t low, high;
      getLoAndHi(timestamp, low, high);

      setHighWord(high);
      return setLowWord(low);
}
//------------------------------------------------------------------------------
// Generate a time stamp
void RawTimingMarker::generate(){
    cycle_t full_timestamp;

    full_timestamp.full_time = tsc();

    setLowWord (full_timestamp.time[0]);
    setHighWord(full_timestamp.time[1]);
}

//------------------------------------------------------------------------------
// Print the Timing Marker
void RawTimingMarker::printStandard(std::ostream& os) const {

    os<<boost::format("\nTiming Marker: HighWord=%u LowWord=%u")%getHighWord()
                                                                %getLowWord();
    
    os<<"  RAW=";
        printWord(0,false, os);
    os<<" ";
        printWord(1,false, os);
    os<<std::endl;
}

//------------------------------------------------------------------------------
// Size of the timing marker in words
uint32_t RawTimingMarker::sizeofdata() const { return RAWTIMINGMARKER_SIZE; }

}// end of namespace daqdataformats
