#include <DatabaseUtils/DAQConfig/DBColumnParams.h>
#include <iostream>

namespace dbutils {
namespace daqconfig {

const std::string DBColumnParams::DCMID_COLUMN_NAME("dcmname");
const std::string DBColumnParams::DCMNAME_COLUMN_NAME("dcmname");
const std::string DBColumnParams::FEBID_COLUMN_NAME("febid");
const std::string DBColumnParams::PIXELID_COLUMN_NAME("pixelid");
const std::string DBColumnParams::HOSTNAME_COLUMN_NAME("hostname");
const std::string DBColumnParams::TDUNAME_COLUMN_NAME("tduname");
const std::string DBColumnParams::TDUDELAYTYPE_COLUMN_NAME("delaytype");

const uint16_t DBColumnParams::LOW_FEB_ID = 0;
const uint16_t DBColumnParams::HIGH_FEB_ID = 63;

const uint16_t DBColumnParams::LOW_PIXEL_ID = 0;
const uint16_t DBColumnParams::HIGH_PIXEL_ID = 31;

/**
 * Constructs a data column parameter object ...
 */
DBColumnParams::DBColumnParams()
{
    _columnKeysForDefaults[DCMNAME_COLUMN_NAME] = "default";
    _columnKeysForDefaults[FEBID_COLUMN_NAME] = "-1";
    _columnKeysForDefaults[PIXELID_COLUMN_NAME] = "-1";
    _columnKeysForDefaults[HOSTNAME_COLUMN_NAME] = "default";
    _columnKeysForDefaults[TDUNAME_COLUMN_NAME] = "default";
    _columnKeysForDefaults[TDUDELAYTYPE_COLUMN_NAME] = "default";
}

/**
 * Returns the single instance of the database column parameter class.
 *
 * @returns the DBColumnParams singleton instance.
 */
DBColumnParams&
DBColumnParams::getInstance()
{
    static DBColumnParams singletonInstance;
    return singletonInstance;
}

/**
 * Fetches the column value that should be used when selecting
 * the default value for a parameter.
 *
 * @param columnName The name of the column.
 * @param columnValue [Output argument] The resulting column value.
 *
 * @returns true if the specified column name was found, false if not.
 */
bool DBColumnParams::
columnKeyToGetDefaultValue(const std::string& columnName,
                           std::string& columnValue)
{
    std::map<std::string, std::string>::const_iterator iter =
        _columnKeysForDefaults.find(columnName);
    if (iter == _columnKeysForDefaults.end()) {return false;}
    columnValue = iter->second;
    return true;
}

};  // end of namespace daqconfig
};  // end of namespace dbutils
